/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ARCGROUPITEM_H
#define ARCGROUPITEM_H

#include <QGraphicsItem>
#include <QList>

#include "Arc.h"
#include "MainWindow.h"

/**
 * @brief The ArcGroupItem class is a QGraphicsItem representation of a group of \link #Arc Arcs\endlink.
 */
class ArcGroupItem : public QGraphicsItem
{
public:
    /**
     * @brief ArcGroupItem Constructs an empty ArcGroupItem with the given parameters
     * @param substring the substring that will be used for its arcs
     * @param color that will be used, this can be overridden by setColour
     * @param mainWindow a pointer to the instance of MainWindow
     */
    ArcGroupItem(QString substring, QColor color, MainWindow* mainWindow);

    /**
     * @brief boundingRect returns the bounding rectangle of all arcs in this group.
     *
     * This is simply a union of the \link #Arc::boundingRect() bounding rectangles of all Arcs\endlink.
     *
     * This is an override from QGraphicsItem
     * @return the bounding rect
     */
    QRectF boundingRect() const override;

    /**
     * @brief paint all Arcs in this group.
     *
     * This is an override from QGraphicsItem
     * @param painter use this painter
     * @param option options, they provide mouse over information etc
     * @param widget unused
     */
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) override;

    /**
     * @brief shape returns an appriximate shape of this arc group.
     *
     * This is used to get a better approximation than the bounding box for intersection testing. It is needed to get a proper mouse over event.
     * The implementation unites all QPainterPaths of the Arcs, which results in an polygon approximation of those.
     *
     * This is an override from QGraphicsItem
     * @return an approximate shape of the QGraphicsItem
     */
    QPainterPath shape() const override;

    /**
     * @brief hoverMoveEvent is called whenever the mouse hovers this QGraphicsItem
     *
     * This is an override from QGraphicsItem
     * @param event provides context information
     */
    void hoverMoveEvent(QGraphicsSceneHoverEvent* event) override;

    /**
     * @brief addArc adds the given Arc to this group
     * @param item the Arc to be added
     */
    void addArc(Arc item);

    /**
     * @brief setColour changes the painting colour of the Arcs
     * @param colour the new colour
     */
    void setColour(QColor colour);

private:
    QString m_substring;
    QColor m_colour;
    QList<Arc> m_items;
    QRectF m_boundingRect;
    QPainterPath m_painterPath;
    MainWindow* m_mainWindow;
};

#endif // ARCGROUPITEM_H
