/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "EssentialMatchingPairs.h"

/**
 * @brief EssentialMatchingPairs::EssentialMatchingPairs Constructor
 * @param text the text where the essential matching pairs should be found
 * @param minSubsequenceLength the minimum length of an essential matching pair
 */
EssentialMatchingPairs::EssentialMatchingPairs(QStringList text, int minSubsequenceLength){
    this->text = text;
    this->minSubsequenceLength = minSubsequenceLength;
    pos = 0;
    node = Node();
    rootNode = Node().constructTree(text);
    rootNode.setNull(false);
    nodeList = new NodeList(rootNode);
    repRegList = new RepetitionRegionList(text, rootNode);
}

/**
 * @brief EssentialMatchingPairs::getMatchingPair returns the essential matching pair in mp1 and mp2
 * @param mp1 first RepetitionRegionInterval of essential matching pair
 * @param mp2 second RepetitionRegionInterval of essential matching pair
 * @return true if there is a essential matching pair, false if not
 */
bool EssentialMatchingPairs::getMatchingPair(RepetitionRegionInterval& mp1, RepetitionRegionInterval& mp2){
    while(true){
        while(node.getLevel() == 0 || pos > node.getStartPosList().length()-2){
            if(nodeList->hasNextNode()){
                node = nodeList->nextNode();
            }
            else{
                return false;
            }
            pos = 0;
        }
        int substringLength = node.getLevel();
        int s1 = node.getStartPos(pos);
        pos++;
        int s2 = node.getStartPos(pos);
        //check if essential matching pair
        if(isEssential(node, s1, s2, substringLength)){
            mp1.setInterval(s1, s1+substringLength-1);
            mp2.setInterval(s2, s2+substringLength-1);
            return true;
        }
    }
}

/**
 * @brief EssentialMatchingPairs::isEssential checks if matching pair is essential
 * @param node the node containing the substring
 * @param s1 the start position of the first interval
 * @param s2 the start position of the second interval
 * @param length the substring length
 * @return true if essential, false if not
 */
bool EssentialMatchingPairs::isEssential(Node node, int s1, int s2, int length){
    RepetitionRegionInterval interval1(s1, s1+length-1);
    RepetitionRegionInterval interval2(s2, s2+length-1);
    QList<RepetitionRegionInterval> repRegions = repRegList->getRepetitionRegionIntervals(s1);
    for(int i=0;i<repRegions.length();i++){
        RepetitionRegionInterval repRegInterval = repRegions.at(i);
        //check if interval is contained in region or intersects it
        if(!repRegInterval.containsInterval(interval2)){
            if(!repRegInterval.intersectsInterval(interval2)){
                return false;
            }
            continue;
        }
        //check if both intervals are consecutive fundamental substrings for repetition region
        if(repRegInterval.isFundamentalSubstring(interval1) && repRegInterval.isFundamentalSubstring(interval2)){
            if(minSubsequenceLength <= repRegInterval.getSubstringLength()){
                return true;
            }
            else{
                return false;
            }
        }
        //check if intervals are contained in the same fundamental substring of repetition region that contains it
        if(!repRegInterval.haveSameFundamentalSubstring(interval1, interval2)){
            return false;
        }
    }

    //check if subsequence is not shorter than threshold
    if(minSubsequenceLength > length){
        return false;
    }

    //check if maximal
    if((s1 > 0 && text.at(s1-1).compare(text.at(s2-1)) == 0) ||
       (s1+length < text.length() && s2+length < text.length() && text.at(s1+length).compare(text.at(s2+length)) == 0)){
        return false;
    }
    while(!node.isNull() && s2+length < text.length()){
        node = node.getChildWithChar(text.at(s2+length));
        s1--;
        if(!node.isNull() && node.startsAt(s1)){
            return false;
        }
        s2++;
    }

    return true;
}
