/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "MainWindow.h"
#include "ui_MainWindow.h"
#include "EssentialMatchingPairs.h"
#include "RepetitionRegionInterval.h"

#include <QGraphicsScene>
#include <QGraphicsLineItem>
#include <QGraphicsTextItem>
#include <QColorDialog>
#include <QFileDialog>
#include <QImage>
#include <QDebug>

#include "Arc.h"
#include "ArcGroupItem.h"

MainWindow::MainWindow() :
    QMainWindow(nullptr),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    resetScene();


    QTransform m(1, 0, 0, -1, 0, 0);
    ui->graphicsView->setTransform(m, false);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::addArc(Arc item)
{
//    qDebug("arc: %f - %f, w=%f, s=%s\n", item->m_start, item->m_end, item->m_width, item->m_substring.toStdString().c_str());
    if(m_arcMap.contains(item.substring())) {
        m_arcMap[item.substring()]->addArc(item);
    }
    else {
        QColor c = m_arcColour;
        c.setAlpha(ui->spinBox_arcAlpha->value());

        ArcGroupItem* agi = new ArcGroupItem(item.substring(), c, this);
        agi->addArc(item);
        m_arcMap.insert(item.substring(), agi);
    }
}

void MainWindow::setArcRange(float max)
{
    m_arcRangeMax = max;
//    qDebug("range: %f - %f\n", min, max);
    float v = ui->doubleSpinBox_baseLineWidth->value() / 2.f;
    m_baseLineItem->setLine(0, -v, max, -v);
    float maxy = fmax(max, 20.f);
    float plusX = maxy*0.1f;
    m_scene->setSceneRect(-plusX, -maxy*0.25f, max+2*plusX, maxy);
}

void MainWindow::resetScene()
{
    delete m_scene;
    m_arcMap.clear();
    m_scene = new QGraphicsScene(this);
    ui->graphicsView->setScene(m_scene);

    m_baseLineItem = m_scene->addLine(0, 0, 1, 0, QPen(m_baseLineColour, ui->doubleSpinBox_baseLineWidth->value()));

    QFont font("SansSerif", 10);    // setting point size to 10 and then scaling by 0.1 because pointsize 1 doesn't work very well
    font.setLetterSpacing(QFont::AbsoluteSpacing, 5);
    font.setFixedPitch(true);
    m_textItem = m_scene->addText("00", font);

    float s = 0.07685f;
    m_textItem->setTransform(QTransform(s,  0, 0,
                                        0, -s, 0,
                                        0,  0, 1));
    m_textItem->setPos(0, -1);
    m_textItem->setDefaultTextColor(m_textColour);
    setArcRange(2);
}

void MainWindow::processText(QString text, int minSubsequenceLength)
{
    m_text = text;
    m_text.replace(QRegExp("\\s"), " ");
    if(m_displayText)
        m_textItem->setPlainText(m_text);
    else
        m_textItem->setPlainText("");

    QStringList list = text.split("");
    list.removeFirst();
    list.removeLast();

    float max = list.length();
    setArcRange(max);

    EssentialMatchingPairs pairs(list, minSubsequenceLength);
    RepetitionRegionInterval i1(0,0);
    RepetitionRegionInterval i2(0,0);

    while(pairs.getMatchingPair(i1,i2)){
        float start = i1.getStart();
        float width = i1.getIntervalLength();
        float end = i2.getEnd()+1;
        QString subString = text.mid(start, width);

        addArc(Arc(start, width, end, subString));
    }
}

void MainWindow::repaintScene()
{
    m_baseLineItem->setPen(QPen(m_baseLineColour, ui->doubleSpinBox_baseLineWidth->value()));
    setArcRange(m_arcRangeMax);
    m_textItem->setDefaultTextColor(m_textColour);

    if(m_displayText)
        m_textItem->setPlainText(m_text);
    else
        m_textItem->setPlainText("");

    QColor c = m_arcColour;
    c.setAlpha(ui->spinBox_arcAlpha->value());

    foreach(ArcGroupItem* item, m_arcMap) {
        item->setColour(c);
    }
    m_scene->invalidate(m_scene->sceneRect());
}

void MainWindow::resizeEvent(QResizeEvent *)
{
    ui->graphicsView->fitInView(m_scene->sceneRect(), Qt::KeepAspectRatio);
}

void MainWindow::on_pushButton_process_clicked()
{
    resetScene();
    processText(ui->textEdit->toPlainText(), ui->spinBox_minSubsequenceLength->value());
    foreach(ArcGroupItem* item, m_arcMap) {
        m_scene->addItem(item);
    }

    ui->graphicsView->fitInView(m_scene->sceneRect(), Qt::KeepAspectRatio);

}

void MainWindow::on_pushButton_export_clicked()
{
    QString path = QFileDialog::getSaveFileName(this, "Export to PNG", QString(), "*.png");
    if(path.length() < 1) return;
    QImage img(ui->spinBox_exportWidth->value(), ui->spinBox_exportWidth->value(), QImage::Format_ARGB32);

    QPainter painter(&img);
    ui->graphicsView->render(&painter);

    img.save(path, "PNG", 100);
}

void MainWindow::on_pushButton_selectArcColour_clicked()
{
    const QColor colour = QColorDialog::getColor(m_arcColour, this, "Select Arc Colour");

    if (colour.isValid()) {
        m_arcColour = colour;
        ui->pushButton_selectArcColour->setPalette(QPalette(colour));
        repaintScene();
    }
}

void MainWindow::on_pushButton_selectBaseLineColour_clicked()
{
    const QColor colour = QColorDialog::getColor(m_baseLineColour, this, "Select Base Line Colour");

    if (colour.isValid()) {
        m_baseLineColour = colour;
        ui->pushButton_selectBaseLineColour->setPalette(QPalette(colour));
        repaintScene();
    }
}

void MainWindow::on_checkBox_clicked(bool checked)
{
    if(checked) {
        connect(ui->spinBox_minSubsequenceLength, SIGNAL(valueChanged(int)), this, SLOT(on_pushButton_process_clicked()));
        connect(ui->textEdit, SIGNAL(textChanged()), this, SLOT(on_pushButton_process_clicked()));
    }
    else {
        disconnect(ui->spinBox_minSubsequenceLength, SIGNAL(valueChanged(int)), this, SLOT(on_pushButton_process_clicked()));
        disconnect(ui->textEdit, SIGNAL(textChanged()), this, SLOT(on_pushButton_process_clicked()));
    }
}

void MainWindow::on_pushButton_selectBackgroundColour_clicked()
{
    const QColor colour = QColorDialog::getColor(m_backgroundColour, this, "Select Background Colour");
    if(colour.isValid()) {
        m_backgroundColour = colour;
        ui->pushButton_selectBackgroundColour->setPalette(QPalette(colour));
        ui->graphicsView->setBackgroundBrush(m_backgroundColour);
    }
}

void MainWindow::on_pushButton_selectTextColour_clicked()
{
    const QColor colour = QColorDialog::getColor(m_textColour, this, "Select Text Colour");
    if(colour.isValid()) {
        m_textColour = colour;
        ui->pushButton_selectTextColour->setPalette(QPalette(colour));
        repaintScene();
    }
}

void MainWindow::on_checkBox_displayText_clicked(bool checked)
{
    m_displayText = checked;
    repaintScene();
}
