/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QMap>

class QGraphicsScene;
class QGraphicsLineItem;
class QGraphicsTextItem;

class QGraphicsView;

class Arc;
class ArcGroupItem;

namespace Ui {
class MainWindow;
}

/**
 * @brief The MainWindow class
 *
 * This is the core class where all functionality is stitched together. %MainWindow.ui is loaded, a QGraphicsScene is setup and all user interaction code
 * is here.
 */
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    /**
     * @brief MainWindow constructs the main window, loads the user interface file (%MainWindow.ui) and sets some default values.
     */
    explicit MainWindow();

    /**
      * @brief Destructs the main window
      */
    ~MainWindow();

    /**
     * @brief addArc add the given arc to the system
     *
     * Finds the correct ArcGroupItem and adds the new arc there.
     * @param item to be added
     */
    void addArc(Arc item);
    /**
     * @brief setArcRange adjusts the size of the QGraphicsScene
     *
     * All Arcs must be within the range 0 - max.
     * @param max the new maximum
     */
    void setArcRange(float max);

    /**
     * @brief resetScene resets the scene before it is repopulated with processText
     */
    void resetScene();

    /**
     * @brief processText process the given text
     *
     * This is the interface between the gui and the string matching algorithm in the background.
     * @param text to be processed
     * @param minSubsequenceLength don't match on substrings shorter than that
     */
    void processText(QString text, int minSubsequenceLength);

public slots:
    /**
     * @brief repaintScene repaints the scene after a colour, base line width etc was changed.
     *
     * Not called, when the text was changed.
     */
    void repaintScene();

protected:
    /**
     * @brief resizeEvent called, when the window is resized.
     *
     * It adjusts the viewport of MyGraphicsView
     */
    void resizeEvent(QResizeEvent*) override;

private slots:
    void on_pushButton_process_clicked();

    void on_pushButton_export_clicked();

    void on_pushButton_selectArcColour_clicked();

    void on_pushButton_selectBaseLineColour_clicked();

    void on_checkBox_clicked(bool checked);

    void on_pushButton_selectBackgroundColour_clicked();

    void on_pushButton_selectTextColour_clicked();

    void on_checkBox_displayText_clicked(bool checked);

private:
    Ui::MainWindow *ui = nullptr;
    QGraphicsScene* m_scene = nullptr;
    QGraphicsLineItem* m_baseLineItem = nullptr;
    QGraphicsTextItem* m_textItem = nullptr;
    QMap<QString, ArcGroupItem*> m_arcMap;
    QColor m_arcColour = QColor(0, 0, 0);
    QColor m_baseLineColour = QColor(0, 0, 0);
    QColor m_textColour = QColor(0, 0, 0);
    QColor m_backgroundColour = QColor(255, 255, 255);
    float m_arcRangeMax = 1;
    bool m_displayText = true;
    QString m_text = "";
};

#endif // MAINWINDOW_H
