/*
 * ArcVis - demo for the UT-Vienna Visualisierung 2 VU course
 * (http://www.cg.tuwien.ac.at/courses/Visualisierung2/)
 * Copyright (C) 2015 Adam Celarek (www.celarek.at)
 * Copyright (C) 2015 Daniel Steinboeck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Node.h"

/**
 * @brief Node::Node Constructor
 */
Node::Node(){
    null = true;
    level = 0;
    this->character = "";
}

/**
 * @brief Node::Node Constructor
 * @param text the character string
 * @param character the character of the node
 */
Node::Node(QStringList text, QString character){
    null = false;
    level = 0;
    this->text = text;
    this->character = character;
    childMap = new QMap<QString, Node>();
}

/**
 * @brief Node::constructTree constructs the tree for a given text
 * @param text the text
 * @return the root node of the tree
 */
Node Node::constructTree(QStringList text){
    Node rootNode(text,"$rootNode$");
    for(int i=0;i<text.length();i++){
        rootNode.startPos << i;
    }
    rootNode.addChildren();
    return rootNode;
}

/**
 * @brief Node::addChild adds a child node to the current node
 * @param child the child node to be add
 */
void Node::addChild(Node& child){
    child.setLevel(getLevel()+1);
    children.append(child);
}

/**
 * @brief Node::getChild returns the child node at the given index
 * @param i the index
 * @return the child node at index i
 */
Node Node::getChild(int i){
    return children.at(i);
}

/**
 * @brief Node::getChildWithChar returns the child node with the given character
 * @param character the character
 * @return the child node with the given character
 */
Node Node::getChildWithChar(QString character){
    return childMap->value(character);
}

/**
 * @brief Node::getChildren returns a list with all children of the current node
 * @return a list with all children of the current node
 */
QList<Node> Node::getChildren(){
    return children;
}

/**
 * @brief Node::setLevel sets the level of the current node
 * @param lvl the level to be set
 */
void Node::setLevel(int lvl){
    level = lvl;
}

/**
 * @brief Node::getLevel returns the level of the current node
 * @return the level of the current node
 */
int Node::getLevel(){
    return level;
}

/**
 * @brief Node::setNull sets the null state of the current node
 * @param b the state to be set
 */
void Node::setNull(bool b){
    null = b;
}

/**
 * @brief Node::isNull checks if current node is null
 * @return true if null, false if not
 */
bool Node::isNull(){
    return null;
}

/**
 * @brief Node::addStartPos adds starting position to the current node
 * @param pos the starting position to add
 */
void Node::addStartPos(int pos){
    startPos.append(pos);
}

/**
 * @brief Node::addStartPosToChild adds starting position to the child node, of the current node, with the given character
 * @param character the character
 * @param pos the starting position to add
 */
void Node::addStartPosToChild(QString character, int pos){
    for(int i=0;i<getChildren().length();i++){
        if(getChild(i).character.compare(character) == 0){
            Node n = getChild(i);
            n.addStartPos(pos);
            children.removeAt(i);
            children.insert(i,n);
        }
    }
}

/**
 * @brief Node::getStartPos returns the starting position at the given index of the list
 * @param i the index
 * @return the starting position at the given index i of the list
 */
int Node::getStartPos(int i){
    return startPos.at(i);
}

/**
 * @brief Node::getStartPosList returns a list of all starting positions of the current node
 * @return a list of all starting positions of the current node
 */
QList<int> Node::getStartPosList(){
    return startPos;
}

/**
 * @brief Node::startsAt checks if the current node starts at the given starting position
 * @param start the starting position
 * @return true if the current node starts at the given starting position, false if not
 */
bool Node::startsAt(int start){
    for(int i=0;i<startPos.length();i++){
        if(start == getStartPos(i)){
            return true;
        }
    }
    return false;
}

/**
 * @brief Node::getCharAt returns the character of the text at the given index
 * @param i the index
 * @return the character of the text at the given index
 */
QString Node::getCharAt(int i){
    if(i >= 0 && i < text.length()){
        return text.at(i);
    }
    else{
        return "";
    }
}

/**
 * @brief Node::addChildren adds children to the current node
 */
void Node::addChildren(){
    int substringLength = getLevel();
    int n = startPos.length();

    //check if possible to add more children
    if(n > 1){
        //add child if none exists with current character, otherwise add startPosition to existing child
        childMap = new QMap<QString, Node>();
        for(int i=0;i<n;i++){
            int start = startPos.at(i);
            int nextPosition = start+substringLength;
            QString nextCharacter = getCharAt(nextPosition);
            if (!childMap->contains(nextCharacter)){
                Node node(text, nextCharacter);
                node.addStartPos(start);
                addChild(node);
                childMap->insert(nextCharacter, node);
            }
            else{
                Node node = childMap->value(nextCharacter);
                node.addStartPos(start);
                childMap->insert(nextCharacter, node);
                addStartPosToChild(nextCharacter, start);
            }
        }

        //add children to each child
        for(int i=0;i<getChildren().length();i++){
            Node child = getChild(i);
            child.addChildren();
            children.removeAt(i);
            children.insert(i,child);
        }
    }
}
